#!/usr/bin/env python
# -*- coding: utf-8 -*-
#This file is part of PyFuriusiConverter. Copyright 2010 Dean Harris (marcus_furius@hotmail.com)
#
#    PyFuriusiConverter is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyFuriusiConverter is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyFuriusiConverter.  If not, see <http://www.gnu.org/licenses/>.

from gettext import gettext as _
import globals
import log
import os
import re
import subprocess
import sys
import time

class converter():
    def __init__(self, source, destination_folder, device):
                
        #Defaults
        self.default_args = '-f mp4 -y -vcodec xvid -maxrate 1000 -qmin 3 -qmax 5 -g 300 -acodec aac'
        self.default_buffersize = 4096
        self.default_audio_bitrate = 128 # this will be automatically multiplied with 1024 for newer ffmpeg versions
        self.default_video_bitrate = 400 # this will be automatically multiplied with 1024 for newer ffmpeg versions
        self.ipod_width = 320.0
        self.ipod_height = 240.0
        self.iphone_width = 480.0
        self.iphone_height = 320.0
        self.nano_width = 376.0
        self.nano_height = 240
        
        self.source = source
        self.source_file_name = ''
        self.destination_folder = destination_folder
        self.device_type = device
        self.source_width = 0.0
        self.source_height = 0.0
        self.source_aspect = 0.0
        self.source_duration = 0.0
        self.output_width = 0.0
        self.output_height = 0.0
        self.output_padding = 0.0
        self.output_file_name = ''
        self.pad_arg1 = ''
        self.pad_arg2 = ''
        
        self.ffmpeg_command = ''
        self.info = None
        self.progress = 0.0
        self.status = ''
        self.processing = False
        

    def get_source_data(self):
        
        self.info = subprocess.Popen(['ffmpeg', '-i', self.source],
                                stdout=subprocess.PIPE,
                                stdin=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
        while True:
            line = self.info.stdout.readline()
            if line == '' and self.info.poll() != None:
                break
            if re.search('Video:.+ (\d+)x(\d+)', line):
                details = re.split('Video:.+ (\d+)x(\d+)', line)
                self.source_width, self.source_height = float(details[1]), float(details[2])
            elif re.search('Duration:\s+(\d+):(\d+):(\d+)\.(\d+)', line):
                details = re.split('Duration:\s+(\d+):(\d+):(\d+)\.(\d+)', line)
                self.source_duration += float(details[1]) * 3600
                self.source_duration += float(details[2]) * 60
                self.source_duration += float(details[3])
                self.source_duration += float(details[4]) / 10
            log.write(globals.conversion_log, line)
            
        try:
            self.source_aspect = self.source_width / self.source_height
        except:
            self.status = _('''Couldn't figure out source_aspect ratio.''')
            log.write((globals.conversion_log, self.status))
            #Use a default
            self.source_aspect = 1.33333333333
                
    def get_target_ratio(self):
        
        if self.device_type == 'iPhone':
            target_width = int(self.iphone_width)
            target_height = int(self.iphone_height)
        elif self.device_type == 'Nano':
            target_width = int(self.nano_width)
            target_height = int(self.nano_height)
        else:
            target_width = int(self.ipod_width)
            target_height = int(self.ipod_height)
        
        
        self.output_width = target_width
        self.output_height = int(self.output_width / self.source_aspect)
        self.output_height -= (self.output_height % 2)
        self.output_padding = int((target_height - self.output_height) / 2.0)
        #self.output_padding -= (self.output_padding % 2)
        self.pad_arg1, self.pad_arg2 = 'padtop', 'padbottom'
        
        if self.output_padding < 0:
            self.output_height = target_height
            self.output_width = int(self.output_height * self.source_aspect)
            self.output_width -= (self.output_width % 2)
            self.output_padding = int((target_width - self.output_width) / 2.0)
            self.output_padding -= (self.output_padding % 2)
            self.pad_arg1, self.pad_arg2 = 'padleft', 'padright'
                        
        print target_width, target_height, self.device_type, self.output_width, self.output_height, self.pad_arg1, self.pad_arg2, self.output_padding

    def build_ffmpeg_command(self):
        
        self.source_file_name = os.path.basename(self.source)
        extension = os.path.splitext(self.source_file_name)[1]
        output_file_name = re.sub(extension, '.mp4', self.source_file_name)
        self.output_file_name = os.path.join(self.destination_folder, output_file_name)
        self.ffmpeg_command = "ffmpeg -i #source_place_holder# %s -bufsize %s -s %sx%s -%s %s -%s %s -ab %s -b %s #target_place_holder#" % (self.default_args, self.default_buffersize, self.output_width, self.output_height, self.pad_arg1, self.output_padding, self.pad_arg2, self.output_padding, self.default_audio_bitrate, self.default_video_bitrate)

    def update_ffmpeg_command(self, old_arg, new_arg, old_command):
        
        # Re-insert the input output place holders
        old_command = re.sub(self.source, '#source_place_holder#', old_command)
        old_command = re.sub(self.output_file_name, '#target_place_holder#', old_command)
        return re.sub(old_arg, new_arg, old_command)
        
    def convert_video(self):
        
        has_errors = True
        current_ffmpeg_command = self.ffmpeg_command
        print self.ffmpeg_command
        self.status = _('Conversion Running')
        while has_errors:
            has_errors = False
            command_array = current_ffmpeg_command.split(' ')
            # Put file names back into the command
            command_array[2] = self.source
            command_array[len(command_array) - 1] = self.output_file_name
            self.info = subprocess.Popen(command_array,
                                    stdout=subprocess.PIPE,
                                    stdin=subprocess.PIPE,
                                    stderr=subprocess.STDOUT)
            self.processing = True
            while True:
                # Ensure we have enough data
                # The ouput of the conversion from ffmpeg is not in nice lines
                # so we need to get it in bite size chunks
                line = str(self.info.stdout.read(256))
                log.write(globals.conversion_log, line)
                
                if line == '' and self.info.poll() != None:
                    break
                if re.search('Unknown.*code.*xvid', line):
                    current_ffmpeg_command = self.update_ffmpeg_command(
                                                           '-vcodec xvid',
                                                           '-vcodec libxvid',
                                                           current_ffmpeg_command)
                    has_errors = True
                    break
                elif re.search('Unknown.*code.*aac', line):
                    current_ffmpeg_command = self.update_ffmpeg_command(
                                                           '-acodec aac',
                                                           '-acodec libfaac',
                                                           current_ffmpeg_command)
                    has_errors = True
                    break
                elif re.search('The bitrate parameter is set too low', line):
                    self.default_audio_bitrate = self.default_audio_bitrate * 1024
                    self.default_video_bitrate = self.default_video_bitrate * 1024
                    self.default_buffersize = self.default_buffersize * 1024
                    
                    current_ffmpeg_command = self.update_ffmpeg_command(
                                                           '-ab \d+',
                                                           '-ab %s' % str(self.default_audio_bitrate),
                                                           current_ffmpeg_command)
                    current_ffmpeg_command = self.update_ffmpeg_command(
                                                           '-b \d+',
                                                           '-b %s' % str(self.default_video_bitrate),
                                                           current_ffmpeg_command)
                    current_ffmpeg_command = self.update_ffmpeg_command(
                                                           '-bufsize \d+',
                                                           '-bufsize %s' % str(self.default_buffersize),
                                                           current_ffmpeg_command)
                    has_errors = True
                    break
                
                elif re.search('time=([^\s]+)', line):
                    details = re.split('time=([^\s]+)', line)
                    self.progress = (float(details[1]) / self.source_duration)
                    
        self.ffmpeg_command = current_ffmpeg_command
        if self.info.returncode == 0:
            self.status = _('Conversion Complete')
        elif self.info.returncode == 1:
            self.status = _('Conversion Failed')
        self.processing = False
        self.progress = 1
        
    def cancel_conversion(self):
        # Cancel the conversion by killing the subprocess
        if self.info != None:
            self.info.kill()
            self.processing = False
            self.info = None
            time.sleep(0.1)
            self.status = _('Conversion Cancelled')
            log.write(globals.conversion_log, _('\nConversion cancelled by user!'))
            
       

