#!/usr/bin/env python
# -*- coding: utf-8 -*-
#This file is part of PyFuriusiConverter. Copyright 2010 Dean Harris (marcus_furius@hotmail.com)
#
#    PyFuriusiConverter is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyFuriusiConverter is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyFuriusiConverter.  If not, see <http://www.gnu.org/licenses/>.

import sys
try:
    import pygtk
    pygtk.require("2.0")
except:
    pass
try:
    import pango
    import gtk
    import gtk.glade
except:
    sys.exit(1)

import aboutbox
import converter
import globals
import thread
import messagebox
import os
import subprocess
import time
from gettext import gettext as _

class main_window:
    drop_targets = [('text/plain', 0, 3), ]
    def __init__(self):
        
        self.is_converting = False
        
        #Set the Glade file
        self.interface = gtk.glade.XML(globals.application_interface, 'main_window')
        HANDLERS_AND_METHODS = {
            'video_storage_drag_data_recieved':self.video_storage_drag_data_recieved,
            'button_start_clicked': self.button_start_clicked,
            'button_clearall_clicked': self.button_clearall_clicked,
            'button_remove_clicked': self.button_remove_clicked,
            'button_add_clicked': self.button_add_clicked,
            'button_about_clicked': self.button_about_clicked,
            'button_browse_clicked':self.button_browse_clicked,
            'destroy': self.destroy}
        self.interface.signal_autoconnect(HANDLERS_AND_METHODS)
        self.set_icons_and_text()
        self.set_storage_lists()
        self.set_video_treeview()
        self.set_default_save_location()
        self.create_settings_directory()
    
    def create_settings_directory(self):

        #Create settings directory if not present
        if not os.path.exists(globals.settings_directory):
            try:
                os.mkdir(globals.settings_directory)
            except OSError, (errno, strerror):
                messagebox.show(self.interface.get_widget('main_window'), _('Error creating settings directory.\nOS error(%s): %s' % (errno, strerror)), gtk.MESSAGE_ERROR)
            except IOError, (errno, strerror):
                messagebox.show(self.interface.get_widget('main_window'), _('Error creating settings directory.\nIO error(%s): %s' % (errno, strerror)), gtk.MESSAGE_ERROR)
            except:
                messagebox.show(self.interface.get_widget('main_window'), _('Error creating settings directory.\nUnexpected error: %s' % (sys.exc_info()[0])), gtk.MESSAGE_ERROR)
    
    def set_default_save_location(self):
        
        video_location = os.path.join(globals.home_directory, 'Videos')
        if os.path.exists(video_location):
            self.interface.get_widget('entry_video_target').set_text(video_location)
        else:
            self.interface.get_widget('entry_video_target').set_text(globals.home_directory)
    
    def set_icons_and_text(self):

        #Set the Icons and Text
        self.interface.get_widget('main_window').set_title('%s %s' % (globals.assembly_title, globals.assembly_version))
        #Allow button images to be shown
        gtk_settings = self.interface.get_widget('main_window').get_settings()
        gtk_settings.set_property("gtk-button-images", True)
        
        self.main_icon = gtk.gdk.pixbuf_new_from_file(globals.main_icon)
        self.interface.get_widget('main_window').set_icon(self.main_icon)
        #Browse Button
        self.browse_button_image = gtk.Image()
        self.browse_button_image.set_from_stock(gtk.STOCK_OPEN, gtk.ICON_SIZE_MENU)
        self.interface.get_widget('button_browse').set_image(self.browse_button_image)
        self.interface.get_widget('button_browse').set_label(_('_Browse...'))
        
        #Add Button
        self.add_button_image = gtk.Image()
        self.add_button_image.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
        self.interface.get_widget('button_add').set_image(self.add_button_image)
        self.interface.get_widget('button_add').set_label(_('A_dd'))
        
        #Remove Button
        self.remove_button_image = gtk.Image()
        self.remove_button_image.set_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
        self.interface.get_widget('button_remove').set_image(self.remove_button_image)
        self.interface.get_widget('button_remove').set_label(_('_Remove'))
        
        #Clear All Button
        self.clear_button_image = gtk.Image()
        self.clear_button_image.set_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU)
        self.interface.get_widget('button_clearall').set_image(self.clear_button_image)
        self.interface.get_widget('button_clearall').set_label(_('C_lear All'))
        
        #Start Button
        self.interface.get_widget('button_start').set_label(_('_Start'))
        
        #Drag Prompt
        self.interface.get_widget('label_drag_prompt').set_label(_('Drag and drop video files below'))
        
        #Convert label
        self.interface.get_widget('label_convert_to').set_label(_('Convert to: '))
        
        #iPod label
        self.interface.get_widget('radiobutton_ipod').set_label(_('iPod'))
        
        #iPhone label
        self.interface.get_widget('radiobutton_iphone').set_label(_('iPhone / Touch'))
        
        #Nano label
        self.interface.get_widget('radiobutton_nano').set_label(_('Nano'))
        
        #Save converted label
        self.interface.get_widget('label_save_converted').set_label(_('Save converted videos to: '))
        
        #Progressbar label
        self.interface.get_widget('progressbar_progress').set_text(_('Add files, then click Start to convert'))
        
        #Button Start label
        self.interface.get_widget('button_start').set_label(_('_Start'))
        
        #Check button
        self.interface.get_widget('checkbutton_show_log').set_label(_('Show FFmpeg log on completion?'))
    
    def set_storage_lists(self):
        
        #Set storage
        self.video_storage = gtk.ListStore(str, str, str) #Mounted image storage
        
    def set_video_treeview(self):

        #Set treeview for images
        self.treeview = self.interface.get_widget('treeview_videos')
        self.treeview.modify_font(pango.FontDescription('sans 8'))
        self.treeview.set_model(self.video_storage)
        #Create columns
        self.column_mount_source = gtk.TreeViewColumn(_('Source Video'), gtk.CellRendererText(), text=0)
        self.column_status = gtk.TreeViewColumn(_('Status'), gtk.CellRendererText(), text=1)
        self.column_completed = gtk.TreeViewColumn('Completed', gtk.CellRendererText(), text=2)
        #Add sorting
        self.column_mount_source.set_sort_column_id(0)
        self.column_status.set_sort_column_id(1)
        #Append columns
        self.treeview.append_column(self.column_mount_source)
        self.treeview.append_column(self.column_status)
        self.treeview.append_column(self.column_completed)
        #Set Drag n Drop
        self.treeview.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.drop_targets, gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY)

    
    def video_storage_drag_data_recieved(self, widget, context, x, y, sel, ttype, time):
        
        if self.is_converting:
            messagebox.show(self.interface.get_widget('main_window'), 
                            _('You cannot add new videos while conversion is in progress. \nPlease cancel the conversion first to change the conversion list.'), 
                            gtk.MESSAGE_INFO)
            return 
        
        files = sel.data.split('\n')
        for file in files:
            if not file:
                continue

            file = file.strip()
            file = file.replace('%20', ' ')
            if file.startswith('file:///') and os.path.isfile(file[7:]):
                self.video_storage.append([file[7:], _('Pending'), str(False)])
    

    def cancel_current_conversion(self):
        
        iter = self.currently_processing_video[3]
        self.currently_processing_process.cancel_conversion()
        self.video_storage.set(iter, 1, _('Cancelled'))
        self.video_storage.set(iter, 2, str(True))
            
    def button_start_clicked(self, data):
        self.abort = False
        if self.interface.get_widget('button_start').get_label() == _('_Start'):
            #Clear the logcfile swo it's fresh
            if os.path.exists(globals.conversion_log):
                os.remove(globals.conversion_log)

            self.interface.get_widget('button_start').set_label(_('_Cancel'))
            #Disable the add, remove and clear all buttons
            self.interface.get_widget('button_add').set_sensitive(False)
            self.interface.get_widget('button_remove').set_sensitive(False)
            self.interface.get_widget('button_clearall').set_sensitive(False)
            self.interface.get_widget('checkbutton_show_log').set_sensitive(False)
            gtk_iteration()
            videos_to_convert = []
            iter = self.video_storage.get_iter_root()
            while (iter):
                #Get device type
                device = 'iPod'
                if self.interface.get_widget('radiobutton_ipod').get_active():
                    device = 'iPod'
                if self.interface.get_widget('radiobutton_iphone').get_active():
                    device = 'iPhone'
                if self.interface.get_widget('radiobutton_nano').get_active():
                    device = 'Nano'
                
                video = [self.video_storage.get_value(iter, 0), 
                         self.interface.get_widget('entry_video_target').get_text(),
                         device,
                         iter]
                videos_to_convert.append(video)
                iter = self.video_storage.iter_next(iter)
            thread.start_new_thread(self.process_videos, (videos_to_convert,))
            
            time.sleep(0.1) # Give the thread the time to start
            
            while self.is_converting:
                if self.abort:
                    break
                gtk_iteration()
                time.sleep(0.1)
                self.interface.get_widget('progressbar_progress').set_fraction(self.currently_processing_process.progress)
                self.interface.get_widget('progressbar_progress').set_text(_('Converting %s') % self.video_storage.get_value(self.currently_processing_video[3], 0))
        else:
            self.abort = True
            self.cancel_current_conversion()
            
        self.interface.get_widget('button_start').set_label(_('_Start'))
        self.interface.get_widget('button_add').set_sensitive(True)
        self.interface.get_widget('button_remove').set_sensitive(True)
        self.interface.get_widget('button_clearall').set_sensitive(True)
        self.interface.get_widget('checkbutton_show_log').set_sensitive(True)
        self.interface.get_widget('progressbar_progress').set_text(_('Add files, then click Start to convert'))
        self.interface.get_widget('progressbar_progress').set_fraction(0)
        if self.interface.get_widget('checkbutton_show_log').get_active():
            if os.path.exists(globals.conversion_log):
                subprocess.Popen(['gnome-text-editor', globals.conversion_log])
            else:
                messagebox.show(self.interface.get_widget('main_window'), _('%s not found.' % (globals.conversion_log)))
            
    def process_videos(self, videos):
        
        self.is_converting = True
        for video in videos:
            self.currently_processing_video = video
            #print self.video_storage.get_value(video[3], 0)
            self.currently_processing_process = converter.converter(self.currently_processing_video[0],
                                                                    self.currently_processing_video[1],
                                                                    self.currently_processing_video[2])
            self.video_storage.set(self.currently_processing_video[3], 1, _('Converting...'))
            self.video_storage.set(self.currently_processing_video[3], 2, str(False))
            self.currently_processing_process.get_source_data()
            #print self.currently_processing_process.source_width, self.currently_processing_process.source_height, self.currently_processing_process.source_aspect, self.currently_processing_process.source_duration
            self.currently_processing_process.get_target_ratio()
            #print self.currently_processing_process.output_height, self.currently_processing_process.output_width, self.currently_processing_process.output_padding, self.currently_processing_process.pad_arg1, self.currently_processing_process.pad_arg2
            self.currently_processing_process.build_ffmpeg_command()
            self.currently_processing_process.convert_video()
            self.video_storage.set(self.currently_processing_video[3], 1, self.currently_processing_process.status)
            self.video_storage.set(self.currently_processing_video[3], 2, str(True))
        self.is_converting = False
             
    def button_clearall_clicked(self, data):
        
        self.video_storage.clear()
        
    def button_remove_clicked(self, data):
        
        #First get the tree view selection
        selected_video = self.treeview.get_selection()
        
        #Get the TreeModel and TreeIter of our selection
        (self.model, self.iter) = selected_video.get_selected()
        if self.iter is not None:
            self.video_storage.remove(self.iter)
    
    def button_add_clicked(self, data):
        
        dialog = gtk.FileChooserDialog(_('Select Source File...'),
                       None,
                       gtk.FILE_CHOOSER_ACTION_OPEN,
                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_current_folder(globals.home_directory)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            # Add mount point and image to image_storage
            self.video_storage.append([dialog.get_filename(), _('Pending'), str(False)])
        dialog.destroy()
    
    def button_browse_clicked(self, data):
        
        dialog = gtk.FileChooserDialog(_('Select Output Destination...'),
                       None,
                       gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_current_folder(globals.home_directory)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.interface.get_widget('entry_video_target').set_text(dialog.get_filename())
        dialog.destroy()
    
    def button_about_clicked(self, data):
        
        aboutbox.show(self.interface.get_widget('main_window'))
        
    def destroy(self, data):
        
        gtk.main_quit()
        
def gtk_iteration():
        
    while gtk.events_pending():
        gtk.main_iteration(False)

if __name__ == '__main__':
    app = main_window()
    gtk.main() 
        

